#!/usr/bin/env python3

""" Find out the most commonly mentioned RFCs which are *not* yet
commented."""

import os
import re

import xml.etree.ElementTree as ElementTree

maximum = 20
debug = 0
count_one_per_file = True
will_never_blog = [
    # Obsoleted by a newer RFC
    821, 822, 1321, 1341, 1541, 1738, 1771, 1991, 2058, 2234, 2368, 2482, 2535, 2401, 2409, 2646, 2915, 2960, 3041, 3056, 3066, 3068, 3095, 3160, 3513, 3731, 4148, 4642,
    # Not interesting enough
    5354
    ]
start_dir = "."
all_dirs = [start_dir + os.sep + "RFC",
            start_dir + os.sep + "ficheslec",
            start_dir + os.sep + "entries"
            ]

files = {}
for directory in all_dirs:
    for xml_file in os.listdir(directory):
        match_xml = re.search("_xml$", xml_file)
        excluded = re.search("NOT-YET", xml_file)
        if match_xml and not excluded:
            files[xml_file] = directory + os.sep + xml_file

not_commented_rfcs = {}
for xml_file in list(files.keys()):
    tree = ElementTree.fromstring(open(files[xml_file]).read())
    if debug > 3:
        print("Examining %s" % xml_file)
    language = tree.attrib.get('{http://www.w3.org/XML/1998/namespace}lang')
    if language is not None:
        language = language.lower()
        if language != "fr" and language[0:2] != 'fr-':
            continue # Ignore texts in other languages, since all the RFC are
              # commented in french
    rfc_elements = tree.iter("rfc")
    if count_one_per_file:
        already_seen = {}
    for rfc in rfc_elements:
        rfc_num = int(rfc.attrib["num"])
        if rfc_num in will_never_blog:
            continue
        if debug > 4:
            print("\t Checking %i (from %s)" % (rfc_num, xml_file))
        if "local" not in rfc.attrib or \
               rfc.attrib["local"] == "false" or \
               rfc.attrib["local"] == 0: # TODO: RelaxNG xsd:boolean can
            # have values 0 and 1???
            # Do not take auto-references into account
            if xml_file == ("%i.rfc_xml" % rfc_num):
                continue
            if count_one_per_file and rfc_num in already_seen:
                continue
            else:
                already_seen[rfc_num] = True
            if debug > 2:
                print("Adding %i to the list (from %s)" % (rfc_num, xml_file))
            if rfc_num not in not_commented_rfcs:
                not_commented_rfcs[rfc_num] = 1
            else:
                not_commented_rfcs[rfc_num] += 1

rfc_sorted = sorted(list(not_commented_rfcs.keys()), key=lambda rfc :not_commented_rfcs[rfc], reverse=True)
for rfc in rfc_sorted[0:maximum]:
    print("RFC %i: %i times" % (rfc, not_commented_rfcs[rfc]))
    
 
