#!/usr/bin/env python3

import os
import sys
import re
import hashlib
import base64
import getopt
import psycopg2
import xml.etree.ElementTree as ElementTree
import xml.parsers.expat # For exception names

import Utils

start = os.environ["HOME"] + os.sep + "/Blog"
dirs = ["RFC", "entries", "ficheslec"]
url_prefix = "http://www.bortzmeyer.org/"
connection_info="dbname=blog"
maximum = None 

def usage(msg=None):
    print("Usage: %s [-a] [-m max] [-b web-dir] [-d blog-dir]" % sys.argv[0], file=sys.stderr)
    if msg is not None:
        print(msg, file=sys.stderr)

def digest(filename):
    content = open(filename).read()
    return base64.urlsafe_b64encode(hashlib.sha256(content.encode()).digest()).decode()

# Defaults
index_all = False
base = None
directory = None
quiet = False
hashes_for_all = False
try:
    optlist, args = getopt.getopt (sys.argv[1:], "hab:d:qm:s",
                               ["help", "all", "base=", "hashesforall", "directory=",
                                "quiet", "max="])
    for option, value in optlist:
        if option == "--help" or option == "-h":
            usage()
            sys.exit(0)
        elif option == "--all" or option == "-a":
            index_all = True
        elif option == "--quiet" or option == "-q":
            quiet = True
        elif option == "--hashesforall" or option == "-s":
            hashes_for_all = True
        elif option == "--base" or option == "-b":
            if directory is not None:
                usage("Cannot have both -d and -b")
                sys.exit(1)
            base = value
        elif option == "--directory" or option == "-d":
            if base is not None:
                usage("Cannot have both -d and -b")
                sys.exit(1)
            directory = value
        elif option == "--max" or option == "-m":
            maximum = int(value)
        else:
            # Should never occur, it is trapped by getopt
            print("Unknown option %s" % option, file=sys.stderr)
            usage()
            sys.exit(1)
except getopt.error as reason:
    usage(reason)
    sys.exit(1)
if len(args) != 0:
    usage()
    sys.exit(1)
if base is None and directory is None:
    directory = start

conn = psycopg2.connect(connection_info)
cursor = conn.cursor()
db_updated = False
significant_changes = False
cursor.execute("BEGIN;")
cursor.execute("SET client_encoding TO 'UTF-8';") 
files = Utils.all_files(base, directory)
if index_all:
    cursor.execute("DELETE FROM Blog.links_articles;")
    cursor.execute("DELETE FROM Blog.links;")
    cursor.execute("DELETE FROM Blog.wikipedia_articles;")
    cursor.execute("DELETE FROM Blog.wikipedia;")
    cursor.execute("DELETE FROM Blog.articles;")
handled = 0
# For every article of the blog
# TODO: index LaTeX and Docbook files, too, but produce a link to the PDF or
# HTML result.
for (file, fullname) in files:
    try:
        tree = ElementTree.parse(fullname)
    except xml.parsers.expat.ExpatError:
        print("Not a XML file: %s" % fullname, file=sys.stderr)
        continue
    except xml.etree.ElementTree.ParseError:
        print("Not a XML file: %s" % fullname, file=sys.stderr)
        continue
    root = tree.getroot()
    try:
        if root.tag == "rfcdesc":
            title = str("RFC " + root.attrib["num"] + ": ")
        else:
            title = str("")
        title += root.attrib['title']
    except KeyError:
        print("No title in %s?" % fullname, file=sys.stderr)
        sys.exit(1)
    published = root.find("date").text
    updated = root.find("update_date")
    if "{http://www.w3.org/XML/1998/namespace}lang" in root.attrib:
        language = root.attrib["{http://www.w3.org/XML/1998/namespace}lang"]
    else:
        language = "fr"
    if updated is not None:
        updated = updated.text
    if not index_all:
        cursor.execute("""SELECT id,updated FROM Blog.Articles 
                                  WHERE filename = %(filename)s;""",
                           {'filename': file})
    if index_all or (cursor.rowcount == 0): # If not already in the DB
        # Add it
        if not quiet:
            print("Inserting %s" % title)
        handled += 1
        body = Utils.text_content_of(root)
        cursor.execute("""INSERT INTO Blog.Articles 
                                       (filename, published,
                                                updated, hash, language, title, body) VALUES
                                   (%(file)s, %(published)s,
                                    %(updated)s, %(hash)s, %(language)s,
                                    %(title)s, %(body)s);""",
                               {'title': title,
                                'published': published, 
                                'updated':  updated,
                                'hash':  digest(fullname),
                                'language': language,
                                'file': file,
                                'body': body})
        cursor.execute("""SELECT id FROM Blog.Articles 
                                  WHERE filename = %(filename)s;""",
                           {'filename': file})
        id_article = cursor.fetchone()[0]
        wp = Utils.wikipedia_of(root)
        for wp_article in wp:
            cursor.execute("""SELECT id FROM Blog.Wikipedia
                              WHERE name = %(name)s AND lang = %(lang)s;""",
                           {'name': wp_article['name'], 'lang': wp_article['lang']})
            if cursor.rowcount == 0:
                cursor.execute("""INSERT INTO Blog.Wikipedia (name, lang) VALUES
                            (%(name)s, %(lang)s);""",
                               {'lang': wp_article['lang'],
                                'name': wp_article['name']})
                cursor.execute("""SELECT id FROM Blog.Wikipedia
                              WHERE name = %(name)s AND lang = %(lang)s;""",
                           {'name': wp_article['name'], 'lang': wp_article['lang']})
                if cursor.rowcount == 0: # Raison complètement inconnue. Normalement, ça ne devrait jamais échouer.
                    print("Cannot register Wikipedia article \"%s\":%s" % (wp_article['name'], wp_article['lang']), file=sys.stderr)
                    continue
            id_wp = cursor.fetchone()[0]
            cursor.execute("""SELECT wikipedia FROM Blog.Wikipedia_Articles
                              WHERE wikipedia = %(id_wp)s AND article = %(id_article)s;""",
                           {'id_wp': id_wp, 'id_article': id_article})
            if cursor.rowcount == 0:
                cursor.execute("""INSERT INTO Blog.Wikipedia_Articles
                                     (wikipedia, article) VALUES
                                         (%(id_wp)s, %(id_article)s);""",
                               {'id_wp': id_wp, 'id_article': id_article})
        links = Utils.link_of(root)
        for link_url in links:
            cursor.execute("""SELECT id FROM Blog.Links
                              WHERE url = %(url)s;""",
                           {'url': link_url})
            if cursor.rowcount == 0:
                cursor.execute("""INSERT INTO Blog.Links (url) VALUES
                            (%(url)s);""",
                               {'url': link_url})
                cursor.execute("""SELECT id FROM Blog.Links WHERE url = %(url)s;""",
                           {'url': link_url})
            id_link = cursor.fetchone()[0]
            cursor.execute("""SELECT link FROM Blog.Links_Articles
                              WHERE link = %(id_link)s AND article = %(id_article)s;""",
                           {'id_link': id_link, 'id_article': id_article})
            if cursor.rowcount == 0:
                cursor.execute("""INSERT INTO Blog.Links_Articles
                                     (link, article) VALUES
                                         (%(id_link)s, %(id_article)s);""",
                               {'id_link': id_link, 'id_article': id_article})
        db_updated = True
        significant_changes = True
    else:
        updated_data = cursor.fetchone()[1]
        if updated_data is None:
            old_updated = "1000-01-01"
        else:
            old_updated = str(updated_data)
        if index_all or \
               (updated is not None and \
               updated > old_updated): # In the DB, but there
                                          # is a newer version
            if not quiet:
                print("Updating %s" % title)
            body = Utils.text_content_of(root)
            cursor.execute("""UPDATE Blog.Articles
                                      SET updated=%(updated)s,
                                          title=%(title)s,
                                          hash=%(hash)s,
                                          body=%(body)s
                                    WHERE filename = %(file)s;""",
                                {'title': title,
                                 'file': file,
                                'published': published, 
                                'updated':  updated, 
                                'hash': digest(fullname),
                                'body': body})
            # TODO: update Blog.Links and Blog.Wikipedia ?
            db_updated = True
            significant_changes = True
        elif hashes_for_all:
            cursor.execute("""UPDATE Blog.Articles
                                      SET hash=%(hash)s
                                    WHERE filename = %(file)s;""",
                                {'file': file,
                                 'hash': digest(fullname)})
            db_updated = True
    if maximum is not None and handled >= maximum:
        break
if not quiet and significant_changes:
    cursor.execute("SELECT count(id) FROM Blog.Articles");
    print("There are %i articles" % int(cursor.fetchone()[0]))
cursor.execute("COMMIT;")
cursor.close()
conn.close()
