DROP FUNCTION Blog.search(TEXT);
DROP FUNCTION Blog.search_plain(TEXT);
DROP FUNCTION Blog.num_uses(INTEGER);
DROP FUNCTION Blog.num_refs(INTEGER);
DROP FUNCTION Blog.num_words(INTEGER);
DROP FUNCTION Blog.num_characters(INTEGER);
DROP FUNCTION Median_value (TEXT);
DROP INDEX Blog.blog_idx;
DROP TABLE Blog.Wikipedia_Articles;
DROP TABLE Blog.Wikipedia;
DROP TABLE Blog.Links_Articles;
DROP TABLE Blog.Links;
DROP TYPE Blog.Results;
DROP TABLE Blog.Articles;
DROP SCHEMA Blog;

CREATE SCHEMA Blog;
CREATE LANGUAGE 'plpgsql';

CREATE TABLE Blog.Articles (id SERIAL UNIQUE NOT NULL,
       inserted TIMESTAMP NOT NULL DEFAULT now(),
       filename TEXT UNIQUE NOT NULL,
       published DATE NOT NULL,
       updated DATE,
       hash TEXT, -- SHA-256 in Base64 with URL encoding, following
		  -- draft-farrell-decade-ni-10.txt
       title TEXT NOT NULL, -- Not unique: for instance, a revision of a 
       -- RFC has the same title as the old one
       language TEXT DEFAULT 'fr',
       body TEXT NOT NULL);

CREATE TABLE Blog.Links  (id SERIAL UNIQUE NOT NULL,
       url TEXT UNIQUE NOT NULL);
CREATE TABLE Blog.Links_Articles (link INTEGER REFERENCES Blog.Links(id),
       article INTEGER REFERENCES Blog.Articles(id));

CREATE TABLE Blog.Wikipedia (id SERIAL UNIQUE NOT NULL,
       name TEXT NOT NULL, lang TEXT, PRIMARY KEY(name, lang));
CREATE TABLE Blog.Wikipedia_Articles (wikipedia INTEGER 
                                 REFERENCES Blog.Wikipedia(id),
       article INTEGER REFERENCES Blog.Articles(id));

-- Number of characters in an article
CREATE FUNCTION Blog.num_characters (INTEGER) RETURNS INTEGER AS
   'SELECT character_length(body) + character_length(title) FROM Blog.Articles 
           WHERE id = $1;'
   LANGUAGE SQL;

-- Number of words in an article
CREATE FUNCTION Blog.num_words (INTEGER) RETURNS INTEGER AS
   'SELECT length(to_tsvector(body)) + length(to_tsvector(title)) 
           FROM Blog.Articles 
           WHERE id = $1;'
   LANGUAGE SQL;

-- Number of references for a link
CREATE FUNCTION Blog.num_uses (INTEGER) RETURNS BIGINT AS
   'SELECT count(article) FROM Blog.Links_Articles WHERE link = $1;'
  LANGUAGE SQL;

-- Number of references for a Wikipedia article
CREATE FUNCTION Blog.num_refs (INTEGER) RETURNS BIGINT AS
   'SELECT count(article) FROM Blog.Wikipedia_Articles WHERE wikipedia = $1;'
  LANGUAGE SQL;

-- Mostly from http://archives.postgresql.org/pgsql-fr-generale/2008-06/msg00003.php
CREATE FUNCTION median_value (TEXT) RETURNS INTEGER AS
'
DECLARE
  curs_w CURSOR (pivot INTEGER) FOR
     SELECT Blog.num_words(id) AS num_w FROM Blog.Articles 
                           ORDER BY num_w OFFSET pivot LIMIT 2;
  curs_c CURSOR (pivot INTEGER) FOR
     SELECT Blog.num_characters(id) AS num_c FROM Blog.Articles 
                           ORDER BY num_c OFFSET pivot LIMIT 2;
  lowval INTEGER;
  highval INTEGER;
  middle INTEGER;
  num_articles INTEGER;
  median INTEGER;
BEGIN
  SELECT count(id) FROM Blog.Articles INTO num_articles;
  IF num_articles % 2 = 0 THEN
    middle := (num_articles / 2) - 1;
  ELSE
    middle := num_articles / 2;
  END IF;
  IF $1 = ''words'' THEN
      OPEN curs_w(middle);
  ELSIF $1 = ''characters'' THEN
      OPEN curs_c(middle);
  ELSE
      RAISE EXCEPTION ''Unknown entity for median count: %'', $1;
  END IF;
  IF num_articles % 2 = 0 THEN
    IF $1 = ''words'' THEN
      FETCH curs_w INTO lowval;
      FETCH curs_w INTO highval;
    ELSIF $1 = ''characters'' THEN
      FETCH curs_c INTO lowval;
      FETCH curs_c INTO highval;
    END IF;
    median := (highval + lowval) / 2;
  ELSE
    IF $1 = ''words'' THEN
      FETCH curs_w INTO lowval;
    ELSIF $1 = ''characters'' THEN
      FETCH curs_c INTO lowval;
    END IF;
    median := lowval;
  END IF;
  IF $1 = ''words'' THEN
    CLOSE curs_w;
  ELSIF $1 = ''characters'' THEN
    CLOSE curs_c;
  END IF;
  RETURN median;
END;
'
LANGUAGE 'plpgsql';

-- Create our own type to make search() easier
CREATE TYPE Blog.Results AS (id INTEGER, filename TEXT, published DATE, updated DATE, title TEXT, 
                        excerpt TEXT,	 
                        body TEXT, rank REAL);

CREATE INDEX blog_idx ON Blog.Articles USING gin(to_tsvector('french', title || body));

-- TODO: returns a ts_headline which is *centered* on the keyword,
-- instead of starting with it. And a longer one, too
CREATE FUNCTION Blog.search (TEXT) RETURNS SETOF Blog.Results AS
   'SELECT id, filename, published, updated, title, 
           ts_headline(''french'', title || body, to_tsquery(''french'',$1), 
                       ''StartSel = _, StopSel = _'') as excerpt,
           body, ts_rank_cd(to_tsvector(''french'', title || body), 
                            to_tsquery(''french'',$1)) as rank 
      FROM Blog.Articles	
         WHERE to_tsvector(''french'', title || body) @@ to_tsquery(''french'',$1) 
         ORDER BY rank DESC;' 
  LANGUAGE SQL;

CREATE FUNCTION Blog.search_plain (TEXT) RETURNS SETOF Blog.Results AS
   'SELECT id, filename, published, updated, title, 
           ts_headline(''french'', title || body, to_tsquery(''french'',$1), 
                       ''StartSel = _, StopSel = _'') as excerpt,
           body, ts_rank_cd(to_tsvector(''french'', title || body), 
                            plainto_tsquery(''french'',$1)) as rank 
      FROM Blog.Articles	
         WHERE to_tsvector(''french'', title || body) @@ plainto_tsquery(''french'',$1) 
         ORDER BY rank DESC;' 
  LANGUAGE SQL;

-- On the machine with the Web service
-- GRANT USAGE ON SCHEMA blog TO apache; -- Or "www-data" on Debian
-- GRANT SELECT ON Blog.Articles TO apache;
