#!/usr/bin/env python3

""" See entries/transformation-texte.entry_xml """

# TODO: does not work properly if the article is in english since it
# will expand some names to a french version

import sys
import re
import os
import subprocess
import getopt

import Utils

# Parameters
replacements_filename = os.environ['HOME'] + "/Blog/schemas/wikipedia-repl.txt"
schemas = os.environ['HOME'] + "/Blog/schemas"
dir = os.environ['HOME'] + "/Blog"
# Default values, can be changed via the command-line
dry_run = False
verbose = False
do_all = False

# TODO: anchors
# TODO: the context? Articles like transformation-xslt include example
# <wikipedia> elements which are mangled :-(
# Do not add the possibility of spaces in the closing tag, it is a
# non-standard dirty hack to avoid spurious transformations
search_re = re.compile("""(.*?)<wikipedia *((( |^ *)name *= *\"([^\"]+)\" *>([^<]+))|( *>([^<]+)))</wikipedia>""",
                       re.MULTILINE | re.DOTALL)

def usage(msg=None):
    sys.stderr.write("Usage: %s [-v ] [-n] [-a] [filename ...]\n" % \
                     sys.argv[0])
    if msg is not None:
        print(file=sys.stderr.write(msg + "\n"))

def validate(filename, klass="rfc"):
    rnv_check = subprocess.Popen(["rnv",
                              "%s/%s.rnc" % (schemas, klass),
                              filename],
                             shell=False,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
    (rnv_output, rnv_error) = rnv_check.communicate()
    result = rnv_check.wait()
    # rnv does not set the return code :-(
    # if result != 0:
    rnv_error = rnv_error.decode()
    if rnv_error != filename + "\n":
        # rnv has some bugs, for instance spurious "attribute ^url
        # with invalid value" when URLs include things like commas. Better double-check
        xmllint_check = subprocess.Popen(["xmllint",
                                          "--noout",
                                          "--relaxng",
                                          "%s/%s.rng" % (schemas, klass),
                                          filename],
                                         shell=False,
                                         stdout=subprocess.PIPE,
                                         stderr=subprocess.PIPE)
        (xmllint_output, xmllint_error) = xmllint_check.communicate()
        result = xmllint_check.wait()
        if result != 0:
            sys.stderr.write("Warning: file %s failed RNV and xmllint validation\n" % filename)
            # OK, both programs agree. But they also have common bugs
            # (for instance they refuse IRI) so, as of today, we are
            # lax:
            xmllint_lax_check = subprocess.Popen(["xmllint",
                                              "--noout",
                                              filename],
                                             shell=False,
                                             stdout=subprocess.PIPE,
                                             stderr=subprocess.PIPE)
            (xmllint_lax_output, xmllint_lax_error) = xmllint_lax_check.communicate()
            result = xmllint_lax_check.wait()
            if result != 0:
                # OK, real error the file is not even well-formed
                raise Exception("XML validation of %s failed: %s" % (filename, rnv_error)) 
try:
    optlist, args = getopt.getopt (sys.argv[1:], "nva",
                               ["dry-run", "verbose", "all"])
    for option, value in optlist:
        if option == "--dry-run" or option == "-n":
            dry_run = True
        elif option == "--verbose" or option == "-v":
            verbose = True
        elif option == "--all" or option == "-a":
            do_all = True
        else:
            # Should never occur, it is trapped by getopt
            sys.stderr.write("Unknown option %s\n" % option)
            usage()
            sys.exit(1)
except getopt.error as reason:
    usage(reason)
    sys.exit(1)

if (not do_all and len(args) < 1) or (do_all and len(args) >= 1):
    usage("Use either -a or give a list of files")
    sys.exit(1)

replacements = {}
replacements_lang = {'fr': {}, 'en': {}}
repl_file = open(replacements_filename, 'r')
for line in repl_file.readlines():
    if re.search("^ *#", line):
        continue
    if re.search("^ *$", line.strip()):
        continue
    match = re.search("^ *\*([a-z\-]+) +(.*)$", line.strip())
    if match:
        language = match.group(1)
        double = re.split(" +", match.group(2), maxsplit=1)
        if language in replacements_lang:
            replacements_lang[language][double[0]] = double[1]
        else:
            replacements_lang[language] = {}
            replacements_lang[language][double[0]] = double[1]
    else:
        double = re.split(" +", line.strip(), maxsplit=1)
        replacements[double[0]] = double[1]
repl_file.close()

if do_all:
    filenames = [x[1] for x in Utils.all_files(None, dir)]
else:
    filenames = args
for filename in filenames:
    if verbose:
        sys.stdout.write("Checking file %s...\n" % filename)
    changed = False
    ifile = open(filename, 'r')
    ofile = open(filename + ".new", 'w')
    xmlroot = Utils.file2DOM(filename)
    language = Utils.lang_of(xmlroot)
    if language is None:
        language = 'fr'
    content = "".join(ifile.readlines())
    match = None
    for match in re.finditer(search_re, content):
        ofile.write(match.group(1))
        if match.group(7) is None:
            name_attr = True
            name = match.group(5)
            text = match.group(6)
        else:
            name_attr = False
            name = match.group(8)
            text = name
        if name in replacements:
            name_attr = True
            changed = True
            newval = replacements[name]
        elif name in replacements_lang[language]:
            name_attr = True
            changed = True
            newval = replacements_lang[language][name]
        else:
            newval = name
        if name_attr:
            ofile.write("""<wikipedia name=\"%s\">%s</wikipedia>""" % \
                         (newval, text))
        else:
            ofile.write("""<wikipedia>%s</wikipedia>""" % \
                         (text))
    ifile.close()
    # Catch the rest
    if match is not None: # Some articles have zero reference to Wikipedia...
        last_char = match.end()
        ofile.write(content[last_char:])
        ofile.close()
        if re.search("\.entry_xml$", filename):
            klass = "entry"
        elif re.search("\.rfc_xml$", filename):
            klass = "rfc"
        elif re.search("\.fiche_xml$", filename):
            klass = "fichelec"
        else:
            sys.stderr.write("Warning: unknown class for file %s" % filename)
            klass = None
        validate(filename + ".new", klass)
    if changed:
        if not dry_run:
            os.rename(filename, filename + ".backup")
            os.rename(filename + ".new", filename)
        if verbose:
            sys.stdout.write("File %s done and validated\n" % filename)
    else:
        if verbose:
            sys.stdout.write("File %s not changed\n" % filename)
