#!/usr/bin/env python3

import os
import sys
import re
import locale
import unicodedata
import urllib.request, urllib.parse, urllib.error
import time
import io

from simpletal import simpleTAL, simpleTALES, simpleTALUtils
import psycopg2

encoding = "utf-8"
base_uses_blog = False
start_id = "tag:bortzmeyer.org,2006-02:Blog/"
start_url = "http://www.bortzmeyer.org"

def escape(ustr):
    """ Escape an Unicode string to keep only characters which are safe for a filename. """
    result = []
    for letter in ustr:
        # Keep only letters and numbers
        if unicodedata.category(letter) in ['Ll', 'Lm', 'Lo', 'Lt', 'Lu', 'Nd', 'Nl', 'No']:
            result.append(letter)
        elif letter == '&':
            result.append('-and-')
        elif letter == '|':
            result.append('-or-')
        else:
            result.append('-')
    return "".join(result)

def hard_escape(ustr):
    """ Escape an Unicode string to keep only ASCII characters. """
    result = []
    for letter in ustr:
        if letter in chrange ('a', 'z') or letter in chrange('A', 'Z') or \
               letter in chrange('0', '9') or letter in ['-', '_', '.']:
            result.append(letter)
        else:
            result.append('-')
    return "".join(result)

def chrange(c1, c2): 
    return [chr(i) for i in range(ord(c1), ord(c2)+1)] 

def to_postgresql(q, default_or = True):
    """ Converts query q (typically an human-entered string with funny
    characters to PostgreSQL FTS engine syntax. By default, the connector is OR."""
    q = separators.sub(" ", q)
    q = q.strip()
    if default_or:
        connector = "|"
    else:
        connector = "&" # AND
    # TODO: test quoted queries, the result is strange. Are they supported by PostgreSQL FTS?
    return re.sub(" +", connector, q)

def application(environ, start_response):
    status = '200 OK'

    context = simpleTALES.Context(allowPythonPath=True)
    context.addGlobal("rqst_or", True)
    form = urllib.parse.parse_qs(environ.get('QUERY_STRING', ''))
    cursor.execute("SELECT count(id) FROM Blog.articles;")
    total_articles = int(cursor.fetchone()[0])
    context.addGlobal("articles", total_articles)
    context.addGlobal("no_result", False)
    query = None
    oformat = "html"
    if "pattern" in form:
        # TODO: do something if there are several keys "pattern"?
        query = form["pattern"][0]
        if "format" in form:
            oformat = form["format"][0].lower()
            if oformat not in ["html", "atom"] :
                # TODO: better error message
                raise Exception("Unsupported format \"%s\"" % oformat)
        context.addGlobal("pattern", query)
        if "default_connector" in form and form["default_connector"][0] == "AND":
            default_is_or = False
            context.addGlobal("rqst_or", False)
        else:
            default_is_or = True
            context.addGlobal("rqst_or", True)
        try:
            cursor.execute("SELECT to_tsquery('french', '%(query)s')", {'query': query})
        except psycopg2.ProgrammingError:
            # Not PostgreSQL FTS syntax, convert it
            query = to_postgresql(query, default_is_or)
            cursor.execute("ROLLBACK;")
        # TODO: if realquery==pattern, set it to None and do not display it
        context.addGlobal("realquery", query)
        context.addGlobal("atomlink", 
                          "/search?format=atom&pattern=" + urllib.parse.quote(query.encode('UTF-8')))
        context.addGlobal("global_id", start_id + "search/" + \
                              urllib.parse.quote(escape(query).encode('UTF-8'), ''))
        context.addGlobal("date", time.strftime("%Y-%m-%dT00:00:00Z", time.gmtime(time.time())))
        context.addGlobal("generator", program + " (Python %s)" % sys.version)
        cursor.execute("""
             SELECT filename,title,excerpt,published,updated FROM Blog.search(%(query)s) LIMIT 50;""",
                       {'query': query})
        # TODO: display published in the results
        results = []
        for result in cursor.fetchall():
            if base_uses_blog:
                link = re.sub("(.*?)\.(entry|fiche|rfc)_xml$", "/\\1.html", result[0])
            else:
                link = re.sub("(.*?)\.xml$", "/\\1.html", result[0])
            url = start_url + link
            if base_uses_blog:
                shortname = re.sub("(.*?)\.(entry|fiche|rfc)_xml$", "\\1", result[0])
            else:
                shortname = re.sub("(.*?)\.xml$", "/\\1", result[0])
            if oformat == 'atom':
                published = str(result[3]) + "T00:00:00Z"
                if result[4] is None:
                    updated = published
                else:
                    updated = str(result[4]) + "T00:00:00Z"
            else: # TODO format it in French for HTML?
                published = result[3]
                if result[4] is None:
                    updated = published
                else:
                    updated = result[4]
            entry_id = start_id + shortname
            results.append({'filename': link, 'url': url,
                            'title': result[1],
                            'excerpt': result[2], 
                           'published': published, 'updated': updated, 
                            'id': entry_id})
        if len(results) == 0:
            context.addGlobal("no_result", True)
        else:
            context.addGlobal("results", results)
        context.addGlobal("num_results", len(results))
    rendered = io.StringIO()
    response_headers = []
    if oformat == "html":
        html_template.expand (context, rendered,
                              outputEncoding=encoding,
                              docType='<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">')
        response_headers.append(('Content-type', 'text/html; charset="%s"' % encoding))
    if oformat == "atom":
        atom_template.expand (context, rendered, outputEncoding=encoding)
        response_headers.append(('Content-type', 'application/atom+xml; charset="%s"' % encoding))
        # TODO: content disposition: Attachment instead of Inline?
    output = rendered.getvalue().encode()
    response_headers.append(('Content-Length', str(len(output))))
    if query is not None:
        response_headers.append(('Content-Disposition', 
                                 "Inline; filename=bortzmeyer-search-%s.%s; filename*=UTF-8''bortzmeyer-search-%s.%s" % \
                                 (hard_escape(query), oformat,
                                  urllib.parse.quote(escape(query).encode('UTF-8'), ''), oformat)))
    # TODO: an Expires field, set as one hour later (we do not index in real-time)
    # TODO: a Server field? Or add something to the one generated by Apache
    start_response(status, response_headers)

    return [output]

# Initialization
locale.setlocale(locale.LC_ALL, "fr_FR.%s" % encoding)
separators = re.compile("\W+", re.UNICODE)
html_page = None
# https://code.google.com/p/modwsgi/wiki/ApplicationIssues
if 'HOME' not in os.environ:
    import pwd
    userhome = pwd.getpwuid(os.getuid()).pw_dir
else:
    userhome = os.environ['HOME']
for template_name in ["/var/www/www.bortzmeyer.org/wsgis/search.xhtml",
                      "/var/www/wsgis-bortzmeyer/search.xhtml",
                      userhome + "/Blog/wsgis/search.xhtml"]:
    if os.path.isfile(template_name):
        html_page = open(template_name)
        break
if html_page is None:
    raise Exception("search.xhtml not found anywhere")
html_template = simpleTAL.compileXMLTemplate(html_page)
atom_page = None
for template_name in ["/var/www/www.bortzmeyer.org/wsgis/search.atom",
                      "/var/www/wsgis-bortzmeyer/search.atom",
                      userhome + "/Blog/wsgis/search.atom"]:
    if os.path.isfile(template_name):
        atom_page = open(template_name)
        break
if atom_page is None:
    raise Exception("search.atom not found anywhere")
atom_template = simpleTAL.compileXMLTemplate(atom_page)

conn = psycopg2.connect("dbname=blog")
cursor = conn.cursor()
cursor.execute("SET client_encoding TO '%s'" % encoding)

program = "search.py" # When running under mod_wsgi, sys.argv does not contain my name

if __name__ == '__main__':
    program = os.path.basename(sys.argv[0])
    base_uses_blog = True
    def local_debug(status, headers):
        print(status)
        print(headers)
    print("".join(application(os.environ, local_debug)))
    
